"""Generated message classes for authztoolkit version v1.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'authztoolkit'


class AnthosServiceMesh(_messages.Message):
  r"""Message describing AnthosServiceMesh based workload object.

  Fields:
    serviceAccount: Immutable. workload ID = IAM Service account
  """

  serviceAccount = _messages.StringField(1)


class AuthztoolkitProjectsLocationsGetRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class AuthztoolkitProjectsLocationsListRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class AuthztoolkitProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class AuthztoolkitProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class AuthztoolkitProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class AuthztoolkitProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class AuthztoolkitProjectsLocationsPoliciesCreateRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsPoliciesCreateRequest object.

  Fields:
    parent: Required. Value for parent. Example:
      projects/{project)/locations/{location}
    policy: A Policy resource to be passed as the request body.
    policyId: Required. The unique ID of the policy to be created.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  parent = _messages.StringField(1, required=True)
  policy = _messages.MessageField('Policy', 2)
  policyId = _messages.StringField(3)
  requestId = _messages.StringField(4)


class AuthztoolkitProjectsLocationsPoliciesDeleteRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsPoliciesDeleteRequest object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class AuthztoolkitProjectsLocationsPoliciesGetRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsPoliciesGetRequest object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class AuthztoolkitProjectsLocationsPoliciesListRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsPoliciesListRequest object.

  Fields:
    filter: Query filter. Filters must adhere to the following rules: *
      Boolean values must be unquoted "true" or "false" string literals. *
      String values must be double-quoted. * Wildcard character("*") is
      limited to use with the has operator (":"), and can be used only at the
      end of a string literal. * Timestamps must be quoted strings in the
      RFC3339 format. Example : filter=create_time>"2022-05-09T22:28:28Z"
      Filters support logical operators - AND, OR, NOT (Note: OR has higher
      precedence than AND)
    orderBy: Criteria for ordering results. Currently supported fields for
      ordering - name and create_time. Example: order_by="name
      desc,create_time desc".
    pageSize: Requested page size. Server may return fewer items than
      requested. The maximum allowed value is 50, values above this will be
      coerced to 50. Default value: 50
    pageToken: Next page token, received from a previous policies.list call.
      When paginating, all other input parameters (except page_token) provided
      to policies.list call must remain the same.
    parent: Required. Parent value for ListPoliciesRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class AuthztoolkitProjectsLocationsPoliciesPatchRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsPoliciesPatchRequest object.

  Fields:
    name: Name of resource
    policy: A Policy resource to be passed as the request body.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Required. Field mask is used to specify the fields to be
      overwritten in the Policy resource by the update. The fields specified
      in the update_mask are relative to the resource, not the full request.
      If the user provides the value '*', all fields will be updated. Note:
      Field masks are not supported to update deeper-levels of policy_spec,
      i.e. partial updates for policy_spec are not supported.
  """

  name = _messages.StringField(1, required=True)
  policy = _messages.MessageField('Policy', 2)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class AuthztoolkitProjectsLocationsTargetAssociationsCreateRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsTargetAssociationsCreateRequest object.

  Fields:
    parent: Required. Value for parent. Example:
      projects/{project)/locations/{location}
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    targetAssociation: A TargetAssociation resource to be passed as the
      request body.
    targetAssociationId: Required. The unique ID of the TargetAssociation to
      be created.
  """

  parent = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  targetAssociation = _messages.MessageField('TargetAssociation', 3)
  targetAssociationId = _messages.StringField(4)


class AuthztoolkitProjectsLocationsTargetAssociationsDeleteRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsTargetAssociationsDeleteRequest object.

  Fields:
    name: Required. Name of the resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class AuthztoolkitProjectsLocationsTargetAssociationsGetRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsTargetAssociationsGetRequest object.

  Fields:
    name: Required. Name of the resource
  """

  name = _messages.StringField(1, required=True)


class AuthztoolkitProjectsLocationsTargetAssociationsListRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsTargetAssociationsListRequest object.

  Fields:
    filter: Optional. Query filter. Filters must adhere to the following
      rules: * Boolean values must be unquoted "true" or "false" string
      literals. * String values must be double-quoted. * Wildcard
      character("*") is limited to use with the has operator (":"), and can be
      used only at the end of a string literal. * Timestamps must be quoted
      strings in the RFC3339 format. Example :
      filter=create_time>"2022-05-09T22:28:28Z" Filters support logical
      operators - AND, OR, NOT (Note: OR has higher precedence than AND)
    orderBy: Optional. Criteria for ordering results. Currently supported
      fields for ordering - name and create_time. Example: order_by="name
      desc,create_time desc".
    pageSize: Optional. Requested page size. Server may return fewer items
      than requested. The maximum allowed value is 50, values above this will
      be coerced to 50. Default value: 50
    pageToken: Optional. Next page token, received from a previous
      targetAssociations.list call. When paginating, all other input
      parameters (except page_token) provided to targetAssociations.list call
      must remain the same.
    parent: Required. Parent value for ListTargetAssociationsRequest
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class AuthztoolkitProjectsLocationsTargetAssociationsPatchRequest(_messages.Message):
  r"""A AuthztoolkitProjectsLocationsTargetAssociationsPatchRequest object.

  Fields:
    name: Identifier. name of resource
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    targetAssociation: A TargetAssociation resource to be passed as the
      request body.
    updateMask: Required. Field mask is used to specify the fields to be
      overwritten in the TargetAssociation resource by the update. The fields
      specified in the update_mask are relative to the resource, not the full
      request. A field will be overwritten if it is in the mask. If the user
      does not provide a mask then all fields will be overwritten.
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)
  targetAssociation = _messages.MessageField('TargetAssociation', 3)
  updateMask = _messages.StringField(4)


class CELPolicy(_messages.Message):
  r"""Authorization policy schema. A policy is composed of a set of Rules that
  specifies the conditions that need to be met in order for an authorization
  decision to be finalized. For example, a policy could specify that all
  requests to a specific IP address or that requests from a specific SPIFFE ID
  must be allowed. Policy can also contain a CEL expression for custom checks.

  Fields:
    ruleBlocks: List of rule blocks.
  """

  ruleBlocks = _messages.MessageField('RuleBlock', 1, repeated=True)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class From(_messages.Message):
  r"""From clause specifies the principals to whom the rule applies.

  Fields:
    principals: List of requesting principal identifiers.
  """

  principals = _messages.StringField(1, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListPoliciesResponse(_messages.Message):
  r"""Response returned by policies.list method.

  Fields:
    nextPageToken: Next page token. Provide this to retrieve the subsequent
      page. When paginating, all other parameters (except page_size) provided
      to policies.list must match the call that provided the page token.
    policies: The list of Policies.
    unreachable: Represents missing potential additional resources.
  """

  nextPageToken = _messages.StringField(1)
  policies = _messages.MessageField('Policy', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class ListTargetAssociationsResponse(_messages.Message):
  r"""Response returned by targetAssociations.list method.

  Fields:
    nextPageToken: Next page token. Provide this to retrieve the subsequent
      page. When paginating, all other parameters (except page_size) provided
      to targetAssociations.list must match the call that provided the page
      token.
    targetAssociations: The list of TargetAssociation
    unreachable: Represents missing potential additional resources.
  """

  nextPageToken = _messages.StringField(1)
  targetAssociations = _messages.MessageField('TargetAssociation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Name of the resource being targeted by this
      operation.
    verb: Output only. Name of the verb executed by the operation. Example:
      create, update, delete etc.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class Policy(_messages.Message):
  r"""Message describing Policy object

  Fields:
    celPolicy: CEL-based authorization policy schema
    createTime: Output only. Create timestamp
    displayName: An arbitrary user-provided name for policy. The display name
      should adhere to the following format. * Must be 6 to 63 characters in
      length. * Can only contain lowercase letters, numbers, and hyphens. *
      Must start with a letter.
    name: Name of resource
    updateTime: Output only. Update timestamp
  """

  celPolicy = _messages.MessageField('CELPolicy', 1)
  createTime = _messages.StringField(2)
  displayName = _messages.StringField(3)
  name = _messages.StringField(4)
  updateTime = _messages.StringField(5)


class PolicyConfig(_messages.Message):
  r"""Message describing PolicyConfig object consisting of policy name and its
  configurations.

  Fields:
    policy: Required. Full policy name. Example:
      projects/{project}/locations/{location}/policies/{policy}
  """

  policy = _messages.StringField(1)


class Rule(_messages.Message):
  r"""Rule is a set of conditions that are evaluated for each incoming
  authorization request.

  Fields:
    displayName: Display name of the rule
    requestFrom: Request origins this rule applies to.
    requestTo: Request destination this rule applies to.
    when: Custom condition for the request.
  """

  displayName = _messages.StringField(1)
  requestFrom = _messages.MessageField('From', 2)
  requestTo = _messages.MessageField('To', 3)
  when = _messages.MessageField('When', 4)


class RuleBlock(_messages.Message):
  r"""RuleBlock holds the action and rule definitions of an authorization
  policy.

  Enums:
    ActionValueValuesEnum: Action type of this policy.

  Fields:
    action: Action type of this policy.
    rules: Rules that must be evaluated for this policy action.
  """

  class ActionValueValuesEnum(_messages.Enum):
    r"""Action type of this policy.

    Values:
      ACTION_UNSPECIFIED: Policy rules with no action type.
      ALLOW: Maps to allow policy rules.
      DENY: Maps to deny policy rules.
    """
    ACTION_UNSPECIFIED = 0
    ALLOW = 1
    DENY = 2

  action = _messages.EnumField('ActionValueValuesEnum', 1)
  rules = _messages.MessageField('Rule', 2, repeated=True)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TargetAssociation(_messages.Message):
  r"""Message describing TargetAssociation object

  Enums:
    EnableAuthorizationDebugLogValueValuesEnum: Optional. Enable the
      generation of authorization debug logs for the target.

  Fields:
    asmWorkload: Immutable. AnthosServiceMesh based workload. Authorization
      Toolkit does not auto configure the authorization settings on the
      workload.
    createTime: Output only. [Output only] Create time stamp
    displayName: Optional. An arbitrary user-provided name for
      TargetAssociation. The display name should adhere to the following
      format. * Must be 6 to 63 characters in length. * Can only contain
      lowercase letters, numbers, and hyphens. * Must start with a letter.
    enableAuthorizationAuditLog: Optional. Enable the generation of
      authorization audit logs for the target.
    enableAuthorizationDebugLog: Optional. Enable the generation of
      authorization debug logs for the target.
    name: Identifier. name of resource
    policies: Optional. List of policies with full policy name and its
      configuration
    updateTime: Output only. [Output only] Update time stamp
  """

  class EnableAuthorizationDebugLogValueValuesEnum(_messages.Enum):
    r"""Optional. Enable the generation of authorization debug logs for the
    target.

    Values:
      LOG_NONE: Disable the authorization debug log.
      LOG_ERROR: Generate authorization debug log only in case the
        authorization result is an error.
      LOG_DENY_AND_ERROR: Generate authorization debug log only in case the
        authorization is denied or the authorization result is an error.
      LOG_ALL: Generate authorization debug log for all the authorization
        results.
    """
    LOG_NONE = 0
    LOG_ERROR = 1
    LOG_DENY_AND_ERROR = 2
    LOG_ALL = 3

  asmWorkload = _messages.MessageField('AnthosServiceMesh', 1)
  createTime = _messages.StringField(2)
  displayName = _messages.StringField(3)
  enableAuthorizationAuditLog = _messages.BooleanField(4)
  enableAuthorizationDebugLog = _messages.EnumField('EnableAuthorizationDebugLogValueValuesEnum', 5)
  name = _messages.StringField(6)
  policies = _messages.MessageField('PolicyConfig', 7, repeated=True)
  updateTime = _messages.StringField(8)


class To(_messages.Message):
  r"""To clause specifies the host(s), method(s), path(s) and port(s) that the
  rule applies to.

  Fields:
    hosts: List of hosts.
    methods: List of HTTP request methods.
    paths: List of request paths.
    ports: List of host ports.
  """

  hosts = _messages.StringField(1, repeated=True)
  methods = _messages.StringField(2, repeated=True)
  paths = _messages.StringField(3, repeated=True)
  ports = _messages.IntegerField(4, repeated=True, variant=_messages.Variant.INT32)


class When(_messages.Message):
  r"""Custom condition the request must satisfy.

  Fields:
    expr: CEL expression to be evaluated.
  """

  expr = _messages.StringField(1)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utility for Data Lineage commands."""

from typing import Any
from apitools.base.py import base_api
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import properties


def GetClient() -> base_api.BaseApiClient:
  """Returns the API client for the datalineage v1 service."""
  return apis.GetClientInstance('datalineage', 'v1')


def GetMessages() -> Any:
  """Returns the messages module for the datalineage v1 service."""
  return apis.GetMessagesModule('datalineage', 'v1')


def GetConfigResourceName(args: Any) -> str:
  """Constructs the full config resource name based on parent flags.

  Args:
    args: An object containing parsed command line arguments. Expected to have
      attributes 'project', 'folder', and 'organization'.

  Returns:
    A string representing the full resource name for the Data Lineage config,
    in the format 'projects/{project}/locations/global/config',
    'folders/{folder}/locations/global/config', or
    'organizations/{organization}/locations/global/config'.
  """
  if not (args.project or args.folder or args.organization):
    # Default to current project if none specified.
    project = properties.VALUES.core.project.GetOrFail()
    return f'projects/{project}/locations/global/config'

  if args.project:
    return f'projects/{args.project}/locations/global/config'
  elif args.folder:
    return f'folders/{args.folder}/locations/global/config'
  else:
    return f'organizations/{args.organization}/locations/global/config'



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a new AlloyDB secondary instance."""


from googlecloudsdk.api_lib.alloydb import api_util
from googlecloudsdk.api_lib.alloydb import instance_operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.alloydb import flags
from googlecloudsdk.command_lib.alloydb import instance_helper
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


# TODO(b/312466999): Change @base.DefaultUniverseOnly to
# @base.UniverseCompatible once b/312466999 is fixed.
# See go/gcloud-cli-running-tpc-tests.
@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class CreateSecondary(base.CreateCommand):
  """Creates a new AlloyDB SECONDARY instance within a given cluster."""

  detailed_help = {
      'DESCRIPTION': '{description}',
      'EXAMPLES': """\
        To create a new secondary instance, run:

          $ {command} my-instance --cluster=my-cluster --region=us-central1
        """,
  }

  @staticmethod
  def Args(parser):
    """Specifies additional command flags.

    Args:
      parser: argparse.Parser: Parser object for command line inputs
    """
    base.ASYNC_FLAG.AddToParser(parser)
    flags.AddCluster(parser, False)
    flags.AddAvailabilityType(parser)
    flags.AddInstance(parser)
    flags.AddRegion(parser)
    flags.AddDatabaseFlags(parser)
    flags.AddSSLMode(parser, default_from_primary=True)
    flags.AddRequireConnectors(parser)
    flags.AddAssignInboundPublicIp(parser)
    flags.AddAuthorizedExternalNetworks(parser)
    flags.AddOutboundPublicIp(parser, show_negated_in_help=True)
    flags.AddAllowedPSCProjects(parser)
    flags.AddPSCNetworkAttachmentUri(parser)
    flags.AddPSCAutoConnections(parser)
    flags.AddAllocatedIPRangeOverride(parser)

    # Connection pooling flags.
    flags.AddEnableConnectionPooling(parser)
    flags.AddConnectionPoolingPoolMode(parser)
    flags.AddConnectionPoolingMinPoolSize(parser)
    flags.AddConnectionPoolingMaxPoolSize(parser)
    flags.AddConnectionPoolingMaxClientConnections(parser)
    flags.AddConnectionPoolingServerIdleTimeout(parser)
    flags.AddConnectionPoolingQueryWaitTimeout(parser)
    flags.AddConnectionPoolingStatsUsers(parser)
    flags.AddConnectionPoolingIgnoreStartupParameters(parser)
    flags.AddConnectionPoolingServerLifetime(parser)
    flags.AddConnectionPoolingClientConnectionIdleTimeout(parser)
    flags.AddConnectionPoolingMaxPreparedStatements(parser)
    labels_util.AddCreateLabelsFlags(parser)

  def ConstructSecondaryCreateRequestFromArgs(
      self, client, alloydb_messages, cluster_ref, args
  ):
    return instance_helper.ConstructSecondaryCreateRequestFromArgsGA(
        client, alloydb_messages, cluster_ref, args
    )

  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.

    Returns:
      ProcessHttpResponse of the request made.
    """
    client = api_util.AlloyDBClient(self.ReleaseTrack())
    alloydb_client = client.alloydb_client
    alloydb_messages = client.alloydb_messages
    cluster_ref = client.resource_parser.Create(
        'alloydb.projects.locations.clusters',
        projectsId=properties.VALUES.core.project.GetOrFail,
        locationsId=args.region,
        clustersId=args.cluster,
    )
    req = self.ConstructSecondaryCreateRequestFromArgs(
        client, alloydb_messages, cluster_ref, args
    )

    op = alloydb_client.projects_locations_clusters_instances.Createsecondary(
        req
    )
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='alloydb.projects.locations.operations'
    )
    log.status.Print('Operation ID: {}'.format(op_ref.Name()))
    if not args.async_:
      instance_operations.Await(
          op_ref, 'Creating secondary instance', self.ReleaseTrack()
      )
    return op


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateSecondaryBeta(CreateSecondary):
  """Creates a new AlloyDB SECONDARY instance within a given cluster."""

  @classmethod
  def Args(cls, parser):
    super(CreateSecondaryBeta, CreateSecondaryBeta).Args(parser)

  def ConstructSecondaryCreateRequestFromArgs(
      self, client, alloydb_messages, cluster_ref, args
  ):
    return instance_helper.ConstructSecondaryCreateRequestFromArgsBeta(
        client, alloydb_messages, cluster_ref, args
    )


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateSecondaryAlpha(CreateSecondaryBeta):
  """Creates a new AlloyDB SECONDARY instance within a given cluster."""

  @classmethod
  def Args(cls, parser):
    super(CreateSecondaryAlpha, CreateSecondaryAlpha).Args(parser)

  def ConstructSecondaryCreateRequestFromArgs(
      self, client, alloydb_messages, cluster_ref, args
  ):
    return instance_helper.ConstructSecondaryCreateRequestFromArgsAlpha(
        client, alloydb_messages, cluster_ref, args
    )



"""Generated message classes for blockchainnodeengine version v1.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'blockchainnodeengine'


class BlockchainNode(_messages.Message):
  r"""A representation of a blockchain node.

  Enums:
    BlockchainTypeValueValuesEnum: Immutable. The blockchain type of the node.
    StateValueValuesEnum: Output only. A status representing the state of the
      node.

  Messages:
    LabelsValue: User-provided key-value pairs.

  Fields:
    blockchainType: Immutable. The blockchain type of the node.
    connectionInfo: Output only. The connection information used to interact
      with a blockchain node.
    createTime: Output only. The timestamp at which the blockchain node was
      first created.
    ethereumDetails: Ethereum-specific blockchain node details.
    labels: User-provided key-value pairs.
    name: Output only. The fully qualified name of the blockchain node. e.g.
      `projects/my-project/locations/us-central1/blockchainNodes/my-node`.
    privateServiceConnectEnabled: Optional. When true, the node is only
      accessible via Private Service Connect; no public endpoints are exposed.
      Otherwise, the node is only accessible via public endpoints. Warning:
      These nodes are deprecated, please use public endpoints instead.
    state: Output only. A status representing the state of the node.
    updateTime: Output only. The timestamp at which the blockchain node was
      last updated.
  """

  class BlockchainTypeValueValuesEnum(_messages.Enum):
    r"""Immutable. The blockchain type of the node.

    Values:
      BLOCKCHAIN_TYPE_UNSPECIFIED: Blockchain type has not been specified, but
        should be.
      ETHEREUM: The blockchain type is Ethereum.
    """
    BLOCKCHAIN_TYPE_UNSPECIFIED = 0
    ETHEREUM = 1

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. A status representing the state of the node.

    Values:
      STATE_UNSPECIFIED: The state has not been specified.
      CREATING: The node has been requested and is in the process of being
        created.
      DELETING: The existing node is undergoing deletion, but is not yet
        finished.
      RUNNING: The node is running and ready for use.
      ERROR: The node is in an unexpected or errored state.
      UPDATING: The node is currently being updated.
      REPAIRING: The node is currently being repaired.
      RECONCILING: The node is currently being reconciled.
      SYNCING: The node is syncing, which is the process by which it obtains
        the latest block and current global state.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    DELETING = 2
    RUNNING = 3
    ERROR = 4
    UPDATING = 5
    REPAIRING = 6
    RECONCILING = 7
    SYNCING = 8

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""User-provided key-value pairs.

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  blockchainType = _messages.EnumField('BlockchainTypeValueValuesEnum', 1)
  connectionInfo = _messages.MessageField('ConnectionInfo', 2)
  createTime = _messages.StringField(3)
  ethereumDetails = _messages.MessageField('EthereumDetails', 4)
  labels = _messages.MessageField('LabelsValue', 5)
  name = _messages.StringField(6)
  privateServiceConnectEnabled = _messages.BooleanField(7)
  state = _messages.EnumField('StateValueValuesEnum', 8)
  updateTime = _messages.StringField(9)


class BlockchainnodeengineProjectsLocationsBlockchainNodesCreateRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsBlockchainNodesCreateRequest
  object.

  Fields:
    blockchainNode: A BlockchainNode resource to be passed as the request
      body.
    blockchainNodeId: Required. ID of the requesting object.
    parent: Required. Value for parent.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  blockchainNode = _messages.MessageField('BlockchainNode', 1)
  blockchainNodeId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  requestId = _messages.StringField(4)


class BlockchainnodeengineProjectsLocationsBlockchainNodesDeleteRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsBlockchainNodesDeleteRequest
  object.

  Fields:
    name: Required. The fully qualified name of the blockchain node to delete.
      e.g. `projects/my-project/locations/us-central1/blockchainNodes/my-
      node`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes after the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
  """

  name = _messages.StringField(1, required=True)
  requestId = _messages.StringField(2)


class BlockchainnodeengineProjectsLocationsBlockchainNodesGetRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsBlockchainNodesGetRequest object.

  Fields:
    name: Required. The fully qualified name of the blockchain node to fetch.
      e.g. `projects/my-project/locations/us-central1/blockchainNodes/my-
      node`.
  """

  name = _messages.StringField(1, required=True)


class BlockchainnodeengineProjectsLocationsBlockchainNodesListRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsBlockchainNodesListRequest
  object.

  Fields:
    filter: Filtering results.
    orderBy: Hint for how to order the results.
    pageSize: Requested page size. Server may return fewer items than
      requested. If unspecified, server will pick an appropriate default.
    pageToken: A token identifying a page of results the server should return.
    parent: Required. Parent value for `ListNodesRequest`.
  """

  filter = _messages.StringField(1)
  orderBy = _messages.StringField(2)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  parent = _messages.StringField(5, required=True)


class BlockchainnodeengineProjectsLocationsBlockchainNodesPatchRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsBlockchainNodesPatchRequest
  object.

  Fields:
    blockchainNode: A BlockchainNode resource to be passed as the request
      body.
    name: Output only. The fully qualified name of the blockchain node. e.g.
      `projects/my-project/locations/us-central1/blockchainNodes/my-node`.
    requestId: Optional. An optional request ID to identify requests. Specify
      a unique request ID so that if you must retry your request, the server
      will know to ignore the request if it has already been completed. The
      server will guarantee that for at least 60 minutes since the first
      request. For example, consider a situation where you make an initial
      request and the request times out. If you make the request again with
      the same request ID, the server can check if original operation with the
      same request ID was received, and if so, will ignore the second request.
      This prevents clients from accidentally creating duplicate commitments.
      The request ID must be a valid UUID with the exception that zero UUID is
      not supported (00000000-0000-0000-0000-000000000000).
    updateMask: Required. Field mask is used to specify the fields to be
      overwritten in the Blockchain node resource by the update. The fields
      specified in the `update_mask` are relative to the resource, not the
      full request. A field will be overwritten if it is in the mask. If the
      user does not provide a mask then all fields will be overwritten.
  """

  blockchainNode = _messages.MessageField('BlockchainNode', 1)
  name = _messages.StringField(2, required=True)
  requestId = _messages.StringField(3)
  updateMask = _messages.StringField(4)


class BlockchainnodeengineProjectsLocationsGetRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class BlockchainnodeengineProjectsLocationsListRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class BlockchainnodeengineProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class BlockchainnodeengineProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class BlockchainnodeengineProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class BlockchainnodeengineProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A BlockchainnodeengineProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class ConnectionInfo(_messages.Message):
  r"""The connection information through which to interact with a blockchain
  node.

  Fields:
    endpointInfo: Output only. The endpoint information through which to
      interact with a blockchain node.
    serviceAttachment: Output only. A service attachment that exposes a node,
      and has the following format: projects/{project}/regions/{region}/servic
      eAttachments/{service_attachment_name}
  """

  endpointInfo = _messages.MessageField('EndpointInfo', 1)
  serviceAttachment = _messages.StringField(2)


class EndpointInfo(_messages.Message):
  r"""Contains endpoint information through which to interact with a
  blockchain node.

  Fields:
    jsonRpcApiEndpoint: Output only. The assigned URL for the node JSON-RPC
      API endpoint.
    websocketsApiEndpoint: Output only. The assigned URL for the node
      WebSockets API endpoint.
  """

  jsonRpcApiEndpoint = _messages.StringField(1)
  websocketsApiEndpoint = _messages.StringField(2)


class EthereumDetails(_messages.Message):
  r"""Ethereum-specific blockchain node details.

  Enums:
    ConsensusClientValueValuesEnum: Immutable. The consensus client.
    ExecutionClientValueValuesEnum: Immutable. The execution client
    NetworkValueValuesEnum: Immutable. The Ethereum environment being
      accessed.
    NodeTypeValueValuesEnum: Immutable. The type of Ethereum node.

  Fields:
    additionalEndpoints: Output only. Ethereum-specific endpoint information.
    apiEnableAdmin: Immutable. Enables JSON-RPC access to functions in the
      `admin` namespace. Defaults to `false`.
    apiEnableDebug: Immutable. Enables JSON-RPC access to functions in the
      `debug` namespace. Defaults to `false`.
    consensusClient: Immutable. The consensus client.
    executionClient: Immutable. The execution client
    gethDetails: Details for the Geth execution client.
    network: Immutable. The Ethereum environment being accessed.
    nodeType: Immutable. The type of Ethereum node.
    validatorConfig: Configuration for validator-related parameters on the
      beacon client, and for any GCP-managed validator client.
  """

  class ConsensusClientValueValuesEnum(_messages.Enum):
    r"""Immutable. The consensus client.

    Values:
      CONSENSUS_CLIENT_UNSPECIFIED: Consensus client has not been specified,
        but should be.
      LIGHTHOUSE: Consensus client implementation written in Rust, maintained
        by Sigma Prime. See [Lighthouse - Sigma
        Prime](https://lighthouse.sigmaprime.io/) for details.
      ERIGON_EMBEDDED_CONSENSUS_LAYER: Erigon's embedded consensus client
        embedded in the execution client. Note this option is not currently
        available when creating new blockchain nodes. See [Erigon on
        GitHub](https://github.com/ledgerwatch/erigon#embedded-consensus-
        layer) for details.
    """
    CONSENSUS_CLIENT_UNSPECIFIED = 0
    LIGHTHOUSE = 1
    ERIGON_EMBEDDED_CONSENSUS_LAYER = 2

  class ExecutionClientValueValuesEnum(_messages.Enum):
    r"""Immutable. The execution client

    Values:
      EXECUTION_CLIENT_UNSPECIFIED: Execution client has not been specified,
        but should be.
      GETH: Official Go implementation of the Ethereum protocol. See [go-
        ethereum](https://geth.ethereum.org/) for details.
      ERIGON: An implementation of Ethereum (execution client), on the
        efficiency frontier, written in Go. See [Erigon on
        GitHub](https://github.com/ledgerwatch/erigon) for details.
    """
    EXECUTION_CLIENT_UNSPECIFIED = 0
    GETH = 1
    ERIGON = 2

  class NetworkValueValuesEnum(_messages.Enum):
    r"""Immutable. The Ethereum environment being accessed.

    Values:
      NETWORK_UNSPECIFIED: The network has not been specified, but should be.
      MAINNET: The Ethereum Mainnet.
      TESTNET_GOERLI_PRATER: Deprecated: The Ethereum Testnet based on Goerli
        protocol. Please use another test network.
      TESTNET_SEPOLIA: The Ethereum Testnet based on Sepolia/Bepolia protocol.
        See https://github.com/eth-clients/sepolia.
      TESTNET_HOLESKY: The Ethereum Testnet based on Holesky specification.
        See https://github.com/eth-clients/holesky.
    """
    NETWORK_UNSPECIFIED = 0
    MAINNET = 1
    TESTNET_GOERLI_PRATER = 2
    TESTNET_SEPOLIA = 3
    TESTNET_HOLESKY = 4

  class NodeTypeValueValuesEnum(_messages.Enum):
    r"""Immutable. The type of Ethereum node.

    Values:
      NODE_TYPE_UNSPECIFIED: Node type has not been specified, but should be.
      LIGHT: An Ethereum node that only downloads Ethereum block headers.
      FULL: Keeps a complete copy of the blockchain data, and contributes to
        the network by receiving, validating, and forwarding transactions.
      ARCHIVE: Holds the same data as full node as well as all of the
        blockchain's history state data dating back to the Genesis Block.
    """
    NODE_TYPE_UNSPECIFIED = 0
    LIGHT = 1
    FULL = 2
    ARCHIVE = 3

  additionalEndpoints = _messages.MessageField('EthereumEndpoints', 1)
  apiEnableAdmin = _messages.BooleanField(2)
  apiEnableDebug = _messages.BooleanField(3)
  consensusClient = _messages.EnumField('ConsensusClientValueValuesEnum', 4)
  executionClient = _messages.EnumField('ExecutionClientValueValuesEnum', 5)
  gethDetails = _messages.MessageField('GethDetails', 6)
  network = _messages.EnumField('NetworkValueValuesEnum', 7)
  nodeType = _messages.EnumField('NodeTypeValueValuesEnum', 8)
  validatorConfig = _messages.MessageField('ValidatorConfig', 9)


class EthereumEndpoints(_messages.Message):
  r"""Contains endpoint information specific to Ethereum nodes.

  Fields:
    beaconApiEndpoint: Output only. The assigned URL for the node's Beacon API
      endpoint.
    beaconPrometheusMetricsApiEndpoint: Output only. The assigned URL for the
      node's Beacon Prometheus metrics endpoint. See [Prometheus
      Metrics](https://lighthouse-book.sigmaprime.io/advanced_metrics.html)
      for more details.
    executionClientPrometheusMetricsApiEndpoint: Output only. The assigned URL
      for the node's execution client's Prometheus metrics endpoint.
  """

  beaconApiEndpoint = _messages.StringField(1)
  beaconPrometheusMetricsApiEndpoint = _messages.StringField(2)
  executionClientPrometheusMetricsApiEndpoint = _messages.StringField(3)


class GethDetails(_messages.Message):
  r"""Options for the Geth execution client. See [Command-line
  Options](https://geth.ethereum.org/docs/fundamentals/command-line-options)
  for more details.

  Enums:
    GarbageCollectionModeValueValuesEnum: Immutable. Blockchain garbage
      collection mode.

  Fields:
    garbageCollectionMode: Immutable. Blockchain garbage collection mode.
  """

  class GarbageCollectionModeValueValuesEnum(_messages.Enum):
    r"""Immutable. Blockchain garbage collection mode.

    Values:
      GARBAGE_COLLECTION_MODE_UNSPECIFIED: The garbage collection has not been
        specified.
      FULL: Configures Geth's garbage collection so that older data not needed
        for a full node is deleted. This is the default mode when creating a
        full node.
      ARCHIVE: Configures Geth's garbage collection so that old data is never
        deleted. This is the default mode when creating an archive node. This
        value can also be chosen when creating a full node in order to create
        a partial/recent archive node. See [Sync
        modes](https://geth.ethereum.org/docs/fundamentals/sync-modes) for
        more details.
    """
    GARBAGE_COLLECTION_MODE_UNSPECIFIED = 0
    FULL = 1
    ARCHIVE = 2

  garbageCollectionMode = _messages.EnumField('GarbageCollectionModeValueValuesEnum', 1)


class GoogleProtobufEmpty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class ListBlockchainNodesResponse(_messages.Message):
  r"""Message for response to listing blockchain nodes.

  Fields:
    blockchainNodes: The list of nodes
    nextPageToken: A token identifying a page of results the server should
      return.
    unreachable: Locations that could not be reached.
  """

  blockchainNodes = _messages.MessageField('BlockchainNode', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have been
      cancelled successfully have `Operation.error` value with a
      `google.rpc.Status.code` of `1`, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class ValidatorConfig(_messages.Message):
  r"""Configuration for validator-related parameters on the beacon client, and
  for any GCP-managed validator client.

  Fields:
    beaconFeeRecipient: An Ethereum address which the beacon client will send
      fee rewards to if no recipient is configured in the validator client.
      See https://lighthouse-book.sigmaprime.io/suggested-fee-recipient.html
      or https://docs.prylabs.network/docs/execution-node/fee-recipient for
      examples of how this is used. Note that while this is often described as
      "suggested", as we run the execution node we can trust the execution
      node, and therefore this is considered enforced.
    managedValidatorClient: Immutable. When true, deploys a GCP-managed
      validator client alongside the beacon client.
    mevRelayUrls: URLs for MEV-relay services to use for block building. When
      set, a GCP-managed MEV-boost service is configured on the beacon client.
  """

  beaconFeeRecipient = _messages.StringField(1)
  managedValidatorClient = _messages.BooleanField(2)
  mevRelayUrls = _messages.StringField(3, repeated=True)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utils for generating API-specific RequestConfig objects.

RequestConfig is provider neutral and should be subclassed into a
provider-specific class (e.g. GcsRequestConfig) by the factory method.

RequestConfig can hold a BucketConfig or ObjectConfig. These classes also
have provider-specific subclasses (e.g. S3ObjectConfig).
"""


from googlecloudsdk.command_lib.storage import encryption_util
from googlecloudsdk.command_lib.storage import errors
from googlecloudsdk.command_lib.storage import storage_url
from googlecloudsdk.core import log
from googlecloudsdk.core.util import debug_output


DEFAULT_CONTENT_TYPE = 'application/octet-stream'


# Bucket update fields and corresponding features unsupported by S3.
S3_REQUEST_ERROR_FIELDS = {
    'gzip_settings': 'Gzip Transforms',
}
S3_RESOURCE_ERROR_FIELDS = {
    'autoclass_terminal_storage_class': (
        'Setting Autoclass Terminal Storage Class'
    ),
    'default_object_acl_file': 'Setting Default Object ACL',
    'enable_autoclass': 'Enabling Autoclass',
    'enable_hierarchical_namespace': 'Enabling Hierarchical Namespace',
    'ip_filter_file_path': 'Setting IP Filter',
    'encryption_enforcement_file_path': 'Setting Encryption Enforcement',
    'predefined_default_object_acl': 'Setting Predefined Default ACL',
    'public_access_prevention': 'Public Access Prevention',
    'recovery_point_objective': 'Setting Recovery Point Objective',
    'retention_period': 'Setting Retention Period',
    'retention_period_to_be_locked': 'Locking Retention Period',
    'custom_contexts_to_set': 'Setting Custom Contexts',
    'custom_contexts_to_remove': 'Removing Custom Contexts',
    'custom_contexts_to_update': 'Updating Custom Contexts',
}
S3_RESOURCE_WARNING_FIELDS = {
    'custom_time': 'Setting Custom Time',
    'default_encryption_key': 'Setting Default Encryption Key',
    'default_event_based_hold': 'Setting Default Event Based Hold',
    'default_storage_class': 'Setting Default Storage Class',
    'enable_per_object_retention': 'Enabling Object Retention',
    'event_based_hold': 'Setting Event-Based Holds',
    'placement': 'Setting Dual-Region for a Bucket',
    'preserve_acl': 'Preserving ACLs',
    'retain_until': 'Setting Time to Retain Until',
    'retention_mode': 'Setting Retention Mode',
    'soft_delete_duration': 'Setting Soft Delete Policies',
    'temporary_hold': 'Setting Temporary Holds',
    'uniform_bucket_level_access': 'Setting Uniform Bucket Level Access',
}


class _ResourceConfig(object):
  """Holder for generic resource fields.

  Attributes:
    acl_file_path (None|str): Path to file with ACL settings.
    acl_grants_to_add (None|list[dict]): Contains API representations of ACL.
      For GCS, this looks like `{ 'entity': ENTITY, 'role': GRANT }`.
    acl_grants_to_remove: (None|list[str]): Identifier of entity to remove
      access for. Can be user, group, project, or keyword like "All".
  """

  def __init__(self,
               acl_file_path=None,
               acl_grants_to_add=None,
               acl_grants_to_remove=None):
    """Initializes class, binding flag values to it."""
    self.acl_file_path = acl_file_path
    self.acl_grants_to_add = acl_grants_to_add
    self.acl_grants_to_remove = acl_grants_to_remove

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (self.acl_file_path == other.acl_file_path and
            self.acl_grants_to_add == other.acl_grants_to_add and
            self.acl_grants_to_remove == other.acl_grants_to_remove)

  def __repr__(self):
    return debug_output.generic_repr(self)


class _BucketConfig(_ResourceConfig):
  """Holder for generic bucket fields.

  More attributes may exist on parent class.

  Attributes:
    cors_file_path (None|str): Path to file with CORS settings.
    labels_file_path (None|str): Path to file with labels settings.
    labels_to_append (None|Dict): Labels to add to a bucket.
    labels_to_remove (None|List[str]): Labels to remove from a bucket.
    lifecycle_file_path (None|str): Path to file with lifecycle settings.
    location (str|None): Location of bucket.
    log_bucket (str|None): Destination bucket for current bucket's logs.
    log_object_prefix (str|None): Prefix for objects containing logs.
    requester_pays (bool|None): If set requester pays all costs related to
      accessing the bucket and its objects.
    versioning (None|bool): Whether to turn on object versioning in a bucket.
    web_error_page (None|str): Error page address if bucket is being used
      to host a website.
    web_main_page_suffix (None|str): Suffix of main page address if bucket is
      being used to host a website.
  """

  def __init__(self,
               acl_file_path=None,
               acl_grants_to_add=None,
               acl_grants_to_remove=None,
               cors_file_path=None,
               labels_file_path=None,
               labels_to_append=None,
               labels_to_remove=None,
               lifecycle_file_path=None,
               location=None,
               log_bucket=None,
               log_object_prefix=None,
               requester_pays=None,
               versioning=None,
               web_error_page=None,
               web_main_page_suffix=None):
    super(_BucketConfig, self).__init__(acl_file_path, acl_grants_to_add,
                                        acl_grants_to_remove)
    self.location = location
    self.cors_file_path = cors_file_path
    self.labels_file_path = labels_file_path
    self.labels_to_append = labels_to_append
    self.labels_to_remove = labels_to_remove
    self.lifecycle_file_path = lifecycle_file_path
    self.log_bucket = log_bucket
    self.log_object_prefix = log_object_prefix
    self.requester_pays = requester_pays
    self.versioning = versioning
    self.web_error_page = web_error_page
    self.web_main_page_suffix = web_main_page_suffix

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (super(_BucketConfig, self).__eq__(other) and
            self.cors_file_path == other.cors_file_path and
            self.labels_file_path == other.labels_file_path and
            self.labels_to_append == other.labels_to_append and
            self.labels_to_remove == other.labels_to_remove and
            self.lifecycle_file_path == other.lifecycle_file_path and
            self.location == other.location and
            self.log_bucket == other.log_bucket and
            self.log_object_prefix == other.log_object_prefix and
            self.requester_pays == other.requester_pays and
            self.versioning == other.versioning and
            self.web_error_page == other.web_error_page and
            self.web_main_page_suffix == other.web_main_page_suffix)


class _GcsBucketConfig(_BucketConfig):
  """Holder for GCS-specific bucket fields.

  See superclass for remaining attributes.

  Subclass Attributes:
    autoclass_terminal_storage_class (str|None): The storage class that
      objects in the bucket eventually transition to if they are not '
      read for a certain length of time.
    default_encryption_key (str|None): A key used to encrypt objects
      added to the bucket.
    default_event_based_hold (bool|None): Determines if event-based holds will
      automatically be applied to new objects in bucket.
    default_object_acl_file_path (str|None): File path to default object ACL
      file.
    default_object_acl_grants_to_add (list[dict]|None): Add default object ACL
      grants to an entity for objects in the bucket.
    default_object_acl_grants_to_remove (list[str]|None): Remove default object
      ACL grants.
    default_storage_class (str|None): Storage class assigned to objects in the
      bucket by default.
    enable_autoclass (bool|None): Enable, disable, or don't do anything to the
      autoclass feature. Autoclass automatically changes object storage class
      based on usage.
    enable_per_object_retention (bool|None): Enable the object retention for the
      bucket.
    enable_hierarchical_namespace (bool|None): Enable heirarchical namespace
    during bucket creation.
    placement (list|None): Dual-region of bucket.
    public_access_prevention (bool|None): Blocks public access to bucket.
      See docs for specifics:
      https://cloud.google.com/storage/docs/public-access-prevention
    recovery_point_objective (str|None): Specifies the replication setting for
      dual-region and multi-region buckets.
    retention_period (int|None): Minimum retention period in seconds for objects
      in a bucket. Attempts to delete an object earlier will be denied.
    soft_delete_duration (int|None): Number of seconds objects are preserved and
      restorable after deletion in a bucket with soft delete enabled.
    uniform_bucket_level_access (bool|None):
      Determines if the IAM policies will apply to every object in bucket.
  """

  def __init__(
      self,
      acl_file_path=None,
      acl_grants_to_add=None,
      acl_grants_to_remove=None,
      autoclass_terminal_storage_class=None,
      cors_file_path=None,
      default_encryption_key=None,
      encryption_enforcement_file_path=None,
      default_event_based_hold=None,
      default_object_acl_file_path=None,
      default_object_acl_grants_to_add=None,
      default_object_acl_grants_to_remove=None,
      default_storage_class=None,
      enable_autoclass=None,
      enable_per_object_retention=None,
      enable_hierarchical_namespace=None,
      ip_filter_file_path=None,
      labels_file_path=None,
      labels_to_append=None,
      labels_to_remove=None,
      lifecycle_file_path=None,
      location=None,
      log_bucket=None,
      log_object_prefix=None,
      placement=None,
      public_access_prevention=None,
      recovery_point_objective=None,
      requester_pays=None,
      retention_period=None,
      retention_period_to_be_locked=None,
      soft_delete_duration=None,
      uniform_bucket_level_access=None,
      versioning=None,
      web_error_page=None,
      web_main_page_suffix=None,
  ):
    super(_GcsBucketConfig, self).__init__(
        acl_file_path,
        acl_grants_to_add,
        acl_grants_to_remove,
        cors_file_path,
        labels_file_path,
        labels_to_append,
        labels_to_remove,
        lifecycle_file_path,
        location,
        log_bucket,
        log_object_prefix,
        requester_pays,
        versioning,
        web_error_page,
        web_main_page_suffix,
    )
    self.autoclass_terminal_storage_class = autoclass_terminal_storage_class
    self.default_encryption_key = default_encryption_key
    self.encryption_enforcement_file_path = encryption_enforcement_file_path
    self.default_event_based_hold = default_event_based_hold
    self.default_object_acl_file_path = default_object_acl_file_path
    self.default_object_acl_grants_to_add = default_object_acl_grants_to_add
    self.default_object_acl_grants_to_remove = (
        default_object_acl_grants_to_remove
    )
    self.default_storage_class = default_storage_class
    self.enable_autoclass = enable_autoclass
    self.enable_per_object_retention = enable_per_object_retention
    self.enable_hierarchical_namespace = enable_hierarchical_namespace
    self.ip_filter_file_path = ip_filter_file_path
    self.placement = placement
    self.public_access_prevention = public_access_prevention
    self.recovery_point_objective = recovery_point_objective
    self.requester_pays = requester_pays
    self.retention_period = retention_period
    self.retention_period_to_be_locked = retention_period_to_be_locked
    self.soft_delete_duration = soft_delete_duration
    self.uniform_bucket_level_access = uniform_bucket_level_access

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        super(_GcsBucketConfig, self).__eq__(other)
        and self.autoclass_terminal_storage_class
        == other.autoclass_terminal_storage_class
        and self.default_encryption_key == other.default_encryption_key
        and self.encryption_enforcement_file_path
        == other.encryption_enforcement_file_path
        and self.default_event_based_hold == other.default_event_based_hold
        and self.default_object_acl_grants_to_add
        == other.default_object_acl_grants_to_add
        and self.default_object_acl_grants_to_remove
        == other.default_object_acl_grants_to_remove
        and self.default_storage_class == other.default_storage_class
        and self.enable_autoclass == other.enable_autoclass
        and self.enable_per_object_retention
        == other.enable_per_object_retention
        and self.enable_hierarchical_namespace
        == other.enable_hierarchical_namespace
        and self.ip_filter_file_path == other.ip_filter_file_path
        and self.placement == other.placement
        and self.public_access_prevention == other.public_access_prevention
        and self.recovery_point_objective == other.recovery_point_objective
        and self.requester_pays == other.requester_pays
        and self.retention_period == other.retention_period
        and self.retention_period_to_be_locked
        == other.retention_period_to_be_locked
        and self.soft_delete_duration == other.soft_delete_duration
        and self.uniform_bucket_level_access
        == other.uniform_bucket_level_access
    )


class _S3BucketConfig(_BucketConfig):
  """Holder for S3-specific bucket fields.

  See superclass for attributes.
  We currently don't support any S3-only fields. This class exists to maintain
  the provider-specific subclass pattern used by the request config factory.
  """


class _ObjectConfig(_ResourceConfig):
  """Holder for storage object settings shared between cloud providers.

  Superclass and provider-specific subclasses may add more attributes.

  Attributes:
    cache_control (str|None): Influences how backend caches requests and
      responses.
    content_disposition (str|None): Information on how content should be
      displayed.
    content_encoding (str|None): How content is encoded (e.g. "gzip").
    content_language (str|None): Content's language (e.g. "en" = "English).
    content_type (str|None): Type of data contained in content (e.g.
      "text/html").
    custom_contexts_to_set (dict|None): Custom contexts fields set by user.
    custom_contexts_to_remove (dict|None): Custom contexts fields to be removed
      by user.
    custom_contexts_to_update (dict|None): Custom contexts field to be added or
      changed by user.
    custom_fields_to_set (dict|None): Custom metadata fields set by user.
    custom_fields_to_remove (dict|None): Custom metadata fields to be removed.
    custom_fields_to_update (dict|None): Custom metadata fields to be added or
      changed.
    decryption_key (encryption_util.EncryptionKey): The key that should be used
      to decrypt information in GCS.
    encryption_key (encryption_util.EncryptionKey|None|CLEAR): The key that
      should be used to encrypt information in GCS or clear encryptions (the
      string user_request_args_factory.CLEAR).
    md5_hash (str|None): MD5 digest to use for validation.
    preserve_acl (bool): Whether or not to preserve existing ACLs on an object
      during a copy or other operation.
    size (int|None): Object size in bytes.
    storage_class (str|None): Storage class for cloud object. If None, will use
      bucket's default.
  """

  def __init__(
      self,
      acl_file_path=None,
      acl_grants_to_add=None,
      acl_grants_to_remove=None,
      cache_control=None,
      content_disposition=None,
      content_encoding=None,
      content_language=None,
      content_type=None,
      custom_contexts_to_set=None,
      custom_contexts_to_remove=None,
      custom_contexts_to_update=None,
      custom_fields_to_set=None,
      custom_fields_to_remove=None,
      custom_fields_to_update=None,
      decryption_key=None,
      encryption_key=None,
      md5_hash=None,
      preserve_acl=None,
      size=None,
      storage_class=None,
  ):
    super(_ObjectConfig, self).__init__(
        acl_file_path, acl_grants_to_add, acl_grants_to_remove
    )
    self.cache_control = cache_control
    self.content_disposition = content_disposition
    self.content_encoding = content_encoding
    self.content_language = content_language
    self.content_type = content_type
    self.custom_contexts_to_set = custom_contexts_to_set
    self.custom_contexts_to_remove = custom_contexts_to_remove
    self.custom_contexts_to_update = custom_contexts_to_update
    self.custom_fields_to_set = custom_fields_to_set
    self.custom_fields_to_remove = custom_fields_to_remove
    self.custom_fields_to_update = custom_fields_to_update
    self.decryption_key = decryption_key
    self.encryption_key = encryption_key
    self.md5_hash = md5_hash
    self.preserve_acl = preserve_acl
    self.size = size
    self.storage_class = storage_class

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        super(_ObjectConfig, self).__eq__(other)
        and self.cache_control == other.cache_control
        and self.content_disposition == other.content_disposition
        and self.content_encoding == other.content_encoding
        and self.content_language == other.content_language
        and self.content_type == other.content_type
        and self.custom_contexts_to_set == other.custom_contexts_to_set
        and self.custom_contexts_to_remove == other.custom_contexts_to_remove
        and self.custom_contexts_to_update == other.custom_contexts_to_update
        and self.custom_fields_to_set == other.custom_fields_to_set
        and self.custom_fields_to_remove == other.custom_fields_to_remove
        and self.custom_fields_to_update == other.custom_fields_to_update
        and self.decryption_key == other.decryption_key
        and self.encryption_key == other.encryption_key
        and self.md5_hash == other.md5_hash
        and self.size == other.size
        and self.preserve_acl == other.preserve_acl
        and self.storage_class == other.storage_class
    )


class _GcsObjectConfig(_ObjectConfig):
  """Arguments object for requests with custom GCS parameters.

  See superclass for additional attributes.

  Attributes:
    event_based_hold (bool|None): An event-based hold should be placed on an
      object.
    custom_time (datetime|None): Custom time user can set.
    retain_until (datetime|None): Time to retain the object until.
    retention_mode (flags.RetentionMode|None|CLEAR): The key that should
      be used to set the retention mode policy in GCS or clear retention (the
      string user_request_args_factory.CLEAR).
    temporary_hold (bool|None): A temporary hold should be placed on an object.
  """
  # pylint:enable=g-missing-from-attributes

  def __init__(
      self,
      acl_file_path=None,
      acl_grants_to_add=None,
      acl_grants_to_remove=None,
      cache_control=None,
      content_disposition=None,
      content_encoding=None,
      content_language=None,
      content_type=None,
      custom_contexts_to_set=None,
      custom_contexts_to_remove=None,
      custom_contexts_to_update=None,
      custom_fields_to_set=None,
      custom_fields_to_remove=None,
      custom_fields_to_update=None,
      custom_time=None,
      decryption_key=None,
      encryption_key=None,
      event_based_hold=None,
      md5_hash=None,
      retain_until=None,
      retention_mode=None,
      size=None,
      temporary_hold=None,
  ):
    super(_GcsObjectConfig, self).__init__(
        acl_file_path=acl_file_path,
        acl_grants_to_add=acl_grants_to_add,
        acl_grants_to_remove=acl_grants_to_remove,
        cache_control=cache_control,
        content_disposition=content_disposition,
        content_encoding=content_encoding,
        content_language=content_language,
        content_type=content_type,
        custom_contexts_to_set=custom_contexts_to_set,
        custom_contexts_to_remove=custom_contexts_to_remove,
        custom_contexts_to_update=custom_contexts_to_update,
        custom_fields_to_set=custom_fields_to_set,
        custom_fields_to_remove=custom_fields_to_remove,
        custom_fields_to_update=custom_fields_to_update,
        decryption_key=decryption_key,
        encryption_key=encryption_key,
        md5_hash=md5_hash,
        size=size)
    self.custom_time = custom_time
    self.event_based_hold = event_based_hold
    self.retain_until = retain_until
    self.retention_mode = retention_mode
    self.temporary_hold = temporary_hold

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (super(_GcsObjectConfig, self).__eq__(other) and
            self.custom_time == other.custom_time and
            self.event_based_hold == other.event_based_hold and
            self.retain_until == other.retain_until and
            self.retention_mode == other.retention_mode and
            self.temporary_hold == other.temporary_hold)


class _S3ObjectConfig(_ObjectConfig):
  """We currently do not support any S3-specific object configurations."""


class _RequestConfig(object):
  """Holder for parameters shared between cloud providers.

  Provider-specific subclasses may add more attributes.

  Attributes:
    predefined_acl_string (str|None): ACL to set on resource.
    predefined_default_object_acl_string (str|None): Default ACL to set on
      resources.
    preserve_posix (bool|None): Whether to apply source POSIX metadata to
      destination.
    preserve_symlinks (bool|None): Whether symlinks should be preserved rather
      than followed.
    resource_args (_BucketConfig|_ObjectConfig|None): Holds settings for a cloud
      resource.
  """

  def __init__(
      self,
      predefined_acl_string=None,
      predefined_default_object_acl_string=None,
      preserve_posix=None,
      preserve_symlinks=None,
      resource_args=None,
  ):
    self.predefined_acl_string = predefined_acl_string
    self.predefined_default_object_acl_string = (
        predefined_default_object_acl_string
    )
    self.preserve_posix = preserve_posix
    self.preserve_symlinks = preserve_symlinks
    self.resource_args = resource_args

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        self.predefined_acl_string == other.predefined_acl_string
        and self.predefined_default_object_acl_string
        == other.predefined_default_object_acl_string
        and self.preserve_posix == other.preserve_posix
        and self.preserve_symlinks == other.preserve_symlinks
        and self.resource_args == other.resource_args
    )

  def __repr__(self):
    return debug_output.generic_repr(self)


# pylint:disable=g-missing-from-attributes
class _GcsRequestConfig(_RequestConfig):
  """Holder for GCS-specific API request parameters.

  See superclass for additional attributes.

  Attributes:
    gzip_settings (user_request_args_factory.GzipSettings): Contains settings
      for gzipping uploaded files.
    no_clobber (bool): Do not copy if destination resource already exists.
    override_unlocked_retention (bool|None): Needed as confirmation for some
      changes to object retention policies.
    precondition_generation_match (int|None): Perform request only if generation
      of target object matches the given integer. Ignored for bucket requests.
    precondition_metageneration_match (int|None): Perform request only if
      metageneration of target object/bucket matches the given integer.
  """
  # pylint:enable=g-missing-from-attributes

  def __init__(
      self,
      gzip_settings=None,
      no_clobber=None,
      override_unlocked_retention=None,
      precondition_generation_match=None,
      precondition_metageneration_match=None,
      predefined_acl_string=None,
      predefined_default_object_acl_string=None,
      resource_args=None,
  ):
    super(_GcsRequestConfig, self).__init__(
        predefined_acl_string=predefined_acl_string,
        predefined_default_object_acl_string=(
            predefined_default_object_acl_string),
        resource_args=resource_args)
    self.gzip_settings = gzip_settings
    self.no_clobber = no_clobber
    self.override_unlocked_retention = override_unlocked_retention
    self.precondition_generation_match = precondition_generation_match
    self.precondition_metageneration_match = precondition_metageneration_match

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        super(_GcsRequestConfig, self).__eq__(other)
        and self.gzip_settings == other.gzip_settings
        and self.no_clobber == other.no_clobber
        and self.override_unlocked_retention
        == other.override_unlocked_retention
        and self.precondition_generation_match
        == other.precondition_generation_match
        and self.precondition_metageneration_match
        == other.precondition_metageneration_match
    )


class _S3RequestConfig(_RequestConfig):
  """Holder for S3-specific API request parameters.

  Currently just meant for use with S3ObjectConfig and S3BucketConfig in
  the parent class "resource_args" field.
  """


def _extract_unsupported_features_from_user_args(user_args, unsupported_fields):
  """Takes user_args and unsupported_fields and returns feature list."""
  result = []
  for field in unsupported_fields:
    if getattr(user_args, field, None) is not None:
      result.append(unsupported_fields[field])
  return sorted(result)


def _check_for_unsupported_s3_fields(user_request_args):
  """Raises error or logs warning if unsupported S3 field present."""
  user_resource_args = getattr(user_request_args, 'resource_args', None)

  # The default value of False would raise an error.
  error_fields_present = _extract_unsupported_features_from_user_args(
      user_request_args, S3_REQUEST_ERROR_FIELDS
  )

  resource_error_fields = _extract_unsupported_features_from_user_args(
      user_resource_args, S3_RESOURCE_ERROR_FIELDS
  )

  # The default value of False for retention_period_to_be_locked raises a false
  # positive. We check and remove it from the error list instead of mutating the
  # user_resource_args object, which prevents a dictionary-size-changed race
  # condition in multithreaded operations.
  if user_resource_args:
    lock_val = getattr(
        user_resource_args, 'retention_period_to_be_locked', None
    )
    if lock_val is not None and not lock_val:
      lock_error_msg = S3_RESOURCE_ERROR_FIELDS['retention_period_to_be_locked']
      if lock_error_msg in resource_error_fields:
        resource_error_fields.remove(lock_error_msg)

  error_fields_present.extend(resource_error_fields)

  if error_fields_present:
    raise errors.Error(
        'Features disallowed for S3: {}'.format(', '.join(error_fields_present))
    )

  warning_fields_present = _extract_unsupported_features_from_user_args(
      user_resource_args, S3_RESOURCE_WARNING_FIELDS
  )
  if warning_fields_present:
    log.warning(
        'Some features do not have S3 support: {}'.format(
            ', '.join(warning_fields_present)
        )
    )


def _get_request_config_resource_args(url,
                                      content_type=None,
                                      decryption_key_hash_sha256=None,
                                      encryption_key=None,
                                      error_on_missing_key=True,
                                      md5_hash=None,
                                      size=None,
                                      user_request_args=None):
  """Generates metadata for API calls to storage buckets and objects."""
  if not isinstance(url, storage_url.CloudUrl):
    return None
  user_resource_args = getattr(user_request_args, 'resource_args', None)
  new_resource_args = None

  if url.is_bucket():
    if url.scheme in storage_url.VALID_CLOUD_SCHEMES:
      if url.scheme == storage_url.ProviderPrefix.GCS:
        new_resource_args = _GcsBucketConfig()
        if user_resource_args:
          new_resource_args.autoclass_terminal_storage_class = (
              user_resource_args.autoclass_terminal_storage_class)
          new_resource_args.default_encryption_key = (
              user_resource_args.default_encryption_key)
          new_resource_args.encryption_enforcement_file_path = (
              user_resource_args.encryption_enforcement_file_path
          )
          new_resource_args.default_event_based_hold = (
              user_resource_args.default_event_based_hold)
          new_resource_args.default_object_acl_file_path = (
              user_resource_args.default_object_acl_file_path)
          new_resource_args.default_object_acl_grants_to_add = (
              user_resource_args.default_object_acl_grants_to_add)
          new_resource_args.default_object_acl_grants_to_remove = (
              user_resource_args.default_object_acl_grants_to_remove)
          new_resource_args.default_storage_class = (
              user_resource_args.default_storage_class)
          new_resource_args.enable_autoclass = (
              user_resource_args.enable_autoclass)
          new_resource_args.enable_per_object_retention = (
              user_resource_args.enable_per_object_retention
          )
          new_resource_args.enable_hierarchical_namespace = (
              user_resource_args.enable_hierarchical_namespace
          )
          new_resource_args.ip_filter_file_path = (
              user_resource_args.ip_filter_file_path
          )
          new_resource_args.placement = user_resource_args.placement
          new_resource_args.public_access_prevention = (
              user_resource_args.public_access_prevention)
          new_resource_args.recovery_point_objective = (
              user_resource_args.recovery_point_objective)
          new_resource_args.retention_period = (
              user_resource_args.retention_period)
          new_resource_args.retention_period_to_be_locked = (
              user_resource_args.retention_period_to_be_locked)
          new_resource_args.soft_delete_duration = (
              user_resource_args.soft_delete_duration
          )
          new_resource_args.uniform_bucket_level_access = (
              user_resource_args.uniform_bucket_level_access)

      elif url.scheme == storage_url.ProviderPrefix.S3:
        new_resource_args = _S3BucketConfig()
        _check_for_unsupported_s3_fields(user_request_args)

    else:
      new_resource_args = _BucketConfig()

    new_resource_args.location = getattr(user_resource_args, 'location', None)
    new_resource_args.cors_file_path = getattr(
        user_resource_args, 'cors_file_path', None)
    new_resource_args.labels_file_path = getattr(
        user_resource_args, 'labels_file_path', None)
    new_resource_args.labels_to_append = getattr(
        user_resource_args, 'labels_to_append', None)
    new_resource_args.labels_to_remove = getattr(
        user_resource_args, 'labels_to_remove', None)
    new_resource_args.lifecycle_file_path = getattr(
        user_resource_args, 'lifecycle_file_path', None)
    new_resource_args.log_bucket = getattr(
        user_resource_args, 'log_bucket', None)
    new_resource_args.log_object_prefix = getattr(
        user_resource_args, 'log_object_prefix', None)
    new_resource_args.requester_pays = getattr(user_resource_args,
                                               'requester_pays', None)
    new_resource_args.versioning = getattr(
        user_resource_args, 'versioning', None)
    new_resource_args.web_error_page = getattr(
        user_resource_args, 'web_error_page', None)
    new_resource_args.web_main_page_suffix = getattr(
        user_resource_args, 'web_main_page_suffix', None)

  elif url.is_object():
    if url.scheme == storage_url.ProviderPrefix.GCS:
      new_resource_args = _GcsObjectConfig()
      if user_resource_args:
        new_resource_args.custom_time = user_resource_args.custom_time
        new_resource_args.event_based_hold = user_resource_args.event_based_hold
        new_resource_args.retain_until = user_resource_args.retain_until
        new_resource_args.retention_mode = user_resource_args.retention_mode
        new_resource_args.temporary_hold = user_resource_args.temporary_hold

    elif url.scheme == storage_url.ProviderPrefix.S3:
      new_resource_args = _S3ObjectConfig()
      _check_for_unsupported_s3_fields(user_request_args)

    else:
      new_resource_args = _ObjectConfig()

    new_resource_args.content_type = content_type
    new_resource_args.md5_hash = md5_hash
    new_resource_args.size = size

    new_resource_args.encryption_key = (
        encryption_key or encryption_util.get_encryption_key())
    if decryption_key_hash_sha256:
      new_resource_args.decryption_key = encryption_util.get_decryption_key(
          decryption_key_hash_sha256, url if error_on_missing_key else None)

    if user_resource_args:
      # User args should override existing settings.
      if user_resource_args.content_type is not None:
        if user_resource_args.content_type:
          new_resource_args.content_type = user_resource_args.content_type
        else:  # Empty string or other falsey value but not completely unset.
          new_resource_args.content_type = DEFAULT_CONTENT_TYPE

      if user_resource_args.md5_hash is not None:
        new_resource_args.md5_hash = user_resource_args.md5_hash

      new_resource_args.cache_control = user_resource_args.cache_control
      new_resource_args.content_disposition = (
          user_resource_args.content_disposition
      )
      new_resource_args.content_encoding = user_resource_args.content_encoding
      new_resource_args.content_language = user_resource_args.content_language
      new_resource_args.custom_fields_to_set = (
          user_resource_args.custom_fields_to_set
      )
      new_resource_args.custom_fields_to_remove = (
          user_resource_args.custom_fields_to_remove
      )
      new_resource_args.custom_fields_to_update = (
          user_resource_args.custom_fields_to_update
      )
      new_resource_args.custom_contexts_to_set = (
          user_resource_args.custom_contexts_to_set
      )
      new_resource_args.custom_contexts_to_remove = (
          user_resource_args.custom_contexts_to_remove
      )
      new_resource_args.custom_contexts_to_update = (
          user_resource_args.custom_contexts_to_update
      )
      new_resource_args.preserve_acl = user_resource_args.preserve_acl

      if user_resource_args.storage_class:
        # Currently, all providers require all caps storage classes.
        new_resource_args.storage_class = (
            user_resource_args.storage_class.upper()
        )

  if new_resource_args and user_resource_args:
    # Fields that apply to all resource types.
    new_resource_args.acl_file_path = user_resource_args.acl_file_path
    new_resource_args.acl_grants_to_add = user_resource_args.acl_grants_to_add
    new_resource_args.acl_grants_to_remove = (
        user_resource_args.acl_grants_to_remove
    )

  return new_resource_args


def get_request_config(
    url,
    content_type=None,
    decryption_key_hash_sha256=None,
    encryption_key=None,
    error_on_missing_key=True,
    md5_hash=None,
    size=None,
    user_request_args=None,
):
  """Generates API-specific RequestConfig. See output classes for arg info."""
  resource_args = _get_request_config_resource_args(
      url, content_type, decryption_key_hash_sha256, encryption_key,
      error_on_missing_key, md5_hash, size, user_request_args)

  if url.scheme == storage_url.ProviderPrefix.GCS:
    request_config = _GcsRequestConfig(resource_args=resource_args)
    if user_request_args:
      request_config.gzip_settings = user_request_args.gzip_settings
      request_config.override_unlocked_retention = (
          user_request_args.override_unlocked_retention
      )
      if user_request_args.no_clobber:
        request_config.no_clobber = user_request_args.no_clobber
      if user_request_args.precondition_generation_match:
        request_config.precondition_generation_match = int(
            user_request_args.precondition_generation_match)
      if user_request_args.precondition_metageneration_match:
        request_config.precondition_metageneration_match = int(
            user_request_args.precondition_metageneration_match)
  elif url.scheme == storage_url.ProviderPrefix.S3:
    request_config = _S3RequestConfig(resource_args=resource_args)
  else:
    request_config = _RequestConfig(resource_args=resource_args)
  request_config.default_object_acl_file_path = getattr(
      user_request_args, 'default_object_acl_file_path', None)
  request_config.predefined_acl_string = getattr(user_request_args,
                                                 'predefined_acl_string', None)
  request_config.predefined_default_object_acl_string = getattr(
      user_request_args, 'predefined_default_object_acl_string', None)
  request_config.preserve_posix = getattr(
      user_request_args, 'preserve_posix', None
  )
  request_config.preserve_symlinks = (
      user_request_args.preserve_symlinks if user_request_args else None
  )

  return request_config



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for deploying containers from Compose file to Cloud Run."""

import os

from googlecloudsdk.api_lib.run import api_enabler
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import docker_util
from googlecloudsdk.command_lib.projects import util as projects_util
from googlecloudsdk.command_lib.run import artifact_registry
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import pretty_print
from googlecloudsdk.command_lib.run import up
from googlecloudsdk.command_lib.run.compose import compose_resource
from googlecloudsdk.command_lib.run.compose import exceptions as compose_exceptions
from googlecloudsdk.command_lib.run.compose import exit_codes
from googlecloudsdk.command_lib.run.compose import tracker as stages
from googlecloudsdk.core import config
from googlecloudsdk.core import log
from googlecloudsdk.core import metrics
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import progress_tracker


DEFAULT_REPO_NAME = 'cloud-run-source-deploy'


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA, base.ReleaseTrack.GA
)
@base.DefaultUniverseOnly
class Up(base.BinaryBackedCommand):
  """Deploy to Cloud Run from compose specification."""

  detailed_help = {
      'DESCRIPTION': """\
          {description}

          The below attribute in the Compose file specifies the service to be
          used as the ingress container in Cloud Run. Only one service can be
          designated as the ingress container.

              x-google-cloudrun:
                ingress-container: true
          """,
      'EXAMPLES': """\
          To deploy a container from the source Compose file on Cloud Run:

              $ {command} compose.yaml

          To deploy to Cloud Run with unauthenticated access:

              $ {command} compose.yaml --allow-unauthenticated
         """,
  }

  @staticmethod
  def CommonArgs(parser):
    flags.AddDeployFromComposeArgument(parser)
    flags.AddRegionArg(parser)
    flags.AddDryRunFlag(parser)
    flags.AddAllowUnauthenticatedFlag(parser)
    build_group = parser.add_mutually_exclusive_group()
    build_group.add_argument(
        '--no-build',
        action='store_true',
        help='Skip building from source if applicable.',
    )
    build_group.add_argument(
        '--build',
        action='store_true',
        help='Force build of images.',
    )

  @staticmethod
  def Args(parser):
    Up.CommonArgs(parser)

  def _ResourceAndTranslateRun(
      self, command_executor, compose_file, repo, project_number, region, args
  ):
    """Handles the resource and translate run logic."""
    release_track = self.ReleaseTrack()
    cfg_dir = config.Paths().global_config_dir
    # We construct out_dir within the gcloud config directory to avoid
    # polluting the user's source codebase with generated files.
    # cfg_dir ensures the path is platform-specific.
    out_dir = os.path.join(cfg_dir, 'surface', 'run', 'compose')
    resource_response = command_executor(
        command='resource',
        compose_file=compose_file,
        region=region,
        out=out_dir,
    )
    error_message = 'Failed to process compose file.'
    if not resource_response.stdout:
      # This should never happen since project is always returned by resource
      # command
      if resource_response.stderr and isinstance(
          resource_response.stderr, list
      ):
        error_message = resource_response.stderr[-1]
      metrics.CustomKeyValue(
          properties.VALUES.metrics.command_name.Get(),
          'resource_error',
          error_message,
      )
      raise compose_exceptions.GoBinaryError(
          error_message, resource_response.exit_code
      )
    try:
      config_obj = compose_resource.ResourcesConfig.from_json(
          resource_response.stdout[0]
      )
      log.debug('Successfully parsed resources config proto.')
      log.debug(f'ResourcesConfig:\n{config_obj}')

      if not config_obj.project:
      # TODO: b/474246517 - Fix error message type.
        raise compose_exceptions.GcloudError(
            'Could not determine project name from compose file.',
            exit_code=exit_codes.PROJECT_NAME_MISSING,
        )

      project_id = properties.VALUES.core.project.Get(required=True)
      required_apis = config_obj.get_required_apis(args.no_build)
      if required_apis:
        api_enabler.check_and_enable_apis(project_id, required_apis)

      with progress_tracker.StagedProgressTracker(
          'Setting up resources...',
          self._AddTrackerStages(config_obj),
          failure_message='Setup failed',
          suppress_output=False,
          success_message='Resource setup complete.',
      ) as tracker:
        resources_config = config_obj.handle_resources(
            region, repo, tracker, no_build=args.no_build, build=args.build
        )
        log.debug('Handled ResourcesConfig:\n%s', resources_config)

      # Serialize the handled config to JSON
      resources_config_json = resources_config.to_json()
      response = command_executor(
          command='translate',
          compose_file=compose_file,
          resources_config=resources_config_json,  # Pass the JSON string
          project_number=project_number,
          region=region,
          out=out_dir,
      )

      if response.stdout:
        translate_result = compose_resource.TranslateResult.from_json(
            response.stdout[0]
        )
        log.debug('Successfully translated resources config to YAML.')
        log.debug(
            'YAML files:\n'
            f'{list(translate_result.services.values()) + list(translate_result.models.values())}'
        )
        for model_yaml in translate_result.models.values():
          compose_resource.deploy_application(
              yaml_file_path=model_yaml,
              region=region,
              args=args,
              release_track=release_track,
          )
        for service_yaml in translate_result.services.values():
          compose_resource.deploy_application(
              yaml_file_path=service_yaml,
              region=region,
              args=args,
              release_track=release_track,
          )
      else:
        if response.stderr and isinstance(response.stderr, list):
          error_message = response.stderr[-1]
        metrics.CustomKeyValue(
            properties.VALUES.metrics.command_name.Get(),
            'translate_error',
            error_message,
        )
        raise compose_exceptions.GoBinaryError(
            error_message, response.exit_code
        )

      return response
    except compose_exceptions.ComposeError:
      raise
    except exceptions.DeploymentFailedError as e:
      raise compose_exceptions.DeployError(
          str(e), exit_codes.DEPLOY_CONTAINER_FAILED_TO_START
      ) from e
    except exceptions.ConfigurationError as e:
      raise compose_exceptions.DeployError(
          str(e), exit_codes.DEPLOY_PERMISSIONS_ERROR
      ) from e
    except exceptions.HttpError as e:
      raise compose_exceptions.DeployError(
          str(e), exit_codes.DEPLOY_API_ERROR
      ) from e
    except Exception as e:
      log.debug(f'Raw output: {resource_response.stdout}')
      # Keep this as the fallback for truly unexpected errors
      raise compose_exceptions.GcloudError(
          str(e), exit_codes.GCLOUD_UNCATEGORIZED_ERROR
      ) from e

  def Run(self, args):
    """Deploy a container from the source Compose file to Cloud Run."""
    log.status.Print('Deploying from Compose to Cloud Run...')
    try:
      region = flags.GetRegion(args, prompt=True)
    except properties.RequiredPropertyError as e:
      raise compose_exceptions.GcloudError(
          str(e), exit_codes.REGION_NOT_SET
      )
    self._SetRegionConfig(region)
    try:
      project = properties.VALUES.core.project.Get(required=True)
    except properties.RequiredPropertyError as e:
      raise compose_exceptions.GcloudError(
          str(e), exit_codes.PROJECT_NOT_SET
      )
    project_number = projects_util.GetProjectNumber(project)
    repo = self._GenerateRepositoryName(
        project,
        region,
    )
    docker_repo = docker_util.DockerRepo(
        project_id=project,
        location_id=region,
        repo_id=DEFAULT_REPO_NAME,
    )

    if artifact_registry.ShouldCreateRepository(
        docker_repo, skip_activation_prompt=True, skip_console_prompt=True
    ):
      self._CreateARRepository(docker_repo)

    command_executor = up.RunComposeWrapper()
    if args.compose_file:
      compose_file = args.compose_file
    else:
      compose_file = self._GetComposeFile()

    response = self._ResourceAndTranslateRun(
        command_executor, compose_file, repo, project_number, region, args
    )
    return response

  def _GenerateRepositoryName(self, project, region):
    """Generate a name for the repository."""
    repository = '{}-docker.pkg.dev'.format(region)
    return '{}/{}/{}'.format(
        repository, project.replace(':', '/'), DEFAULT_REPO_NAME
    )

  def _SetRegionConfig(self, region):
    """Set the region config."""
    if not properties.VALUES.run.region.Get():
      log.status.Print(
          'Region set to {region}.You can change the region with gcloud'
          ' config set run/region {region}.\n'.format(region=region)
      )
      properties.VALUES.run.region.Set(region)

  def _GetComposeFile(self):
    for filename in [
        'compose.yaml',
        'compose.yml',
        'docker-compose.yaml',
        'docker-compose.yml',
    ]:
      if os.path.exists(filename):
        return filename
    raise exceptions.ConfigurationError(
        'No compose file found. Please provide a compose file.',
        exit_code=exit_codes.COMPOSE_FILE_NOT_FOUND,
    )

  def _CreateARRepository(self, docker_repo):
    """Create an Artifact Registry Repository if not present."""
    pretty_print.Success(
        f'Creating AR Repository in the region: {docker_repo.location}'
    )
    artifact_registry.CreateRepository(docker_repo, skip_activation_prompt=True)

  def _AddTrackerStages(self, cfg):
    """Add a tracker to the progress tracker."""
    staged_operations = []
    if cfg.source_builds:
      for container_name in cfg.source_builds:
        staged_operations.append(
            progress_tracker.Stage(
                f'Building container {container_name} from source...',
                key=stages.StagedProgressTrackerStage.BUILD.get_key(
                    container=container_name
                ),
            )
        )
    if cfg.secrets:
      staged_operations.append(
          progress_tracker.Stage(
              'Creating secrets...',
              key=stages.StagedProgressTrackerStage.SECRETS.get_key(),
          )
      )
    if cfg.volumes.bind_mount or cfg.volumes.named_volume:
      staged_operations.append(
          progress_tracker.Stage(
              'Creating volumes...',
              key=stages.StagedProgressTrackerStage.VOLUMES.get_key(),
          )
      )
    if cfg.configs:
      staged_operations.append(
          progress_tracker.Stage(
              'Creating configs...',
              key=stages.StagedProgressTrackerStage.CONFIGS.get_key(),
          )
      )
    return staged_operations



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Migrate command."""


from googlecloudsdk.api_lib.app import appengine_api_client
from googlecloudsdk.api_lib.app import operations_util
from googlecloudsdk.api_lib.app import service_util
from googlecloudsdk.calliope import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


class VersionsMigrateError(exceptions.Error):
  """Errors when migrating versions."""


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class Migrate(base.Command):
  """Migrate traffic from one version to another for a set of services."""

  detailed_help = {
      'EXAMPLES': """\
          This only works for automatically scaled Standard versions.
          To migrate from one version to another for all services where there
          is a version v2 and shut down the previous version, run:

            $ {command} v2

          To migrate from one version to another for a specific service, run:

            $ {command} v2 --service="s1"
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('version', help='The version to migrate to.')
    parser.add_argument('--service', '-s',
                        help='If specified, only migrate versions from the '
                             'given service.')

  def Run(self, args):
    client = appengine_api_client.GetApiClientForTrack(self.ReleaseTrack())
    if args.service:
      service = client.GetServiceResource(args.service)
      traffic_split = {}
      if service.split:
        for split in service.split.allocations.additionalProperties:
          traffic_split[split.key] = split.value
      services = [service_util.Service(client.project, service.id,
                                       traffic_split)]
    else:
      services = client.ListServices()
    all_versions = client.ListVersions(services)
    if args.version not in {v.id for v in all_versions}:
      if args.service:
        raise VersionsMigrateError('Version [{0}/{1}] does not exist.'.format(
            args.service, args.version))
      else:
        raise VersionsMigrateError('Version [{0}] does not exist.'.format(
            args.version))
    service_names = {v.service for v in all_versions if v.id == args.version}

    def WillBeMigrated(v):
      return (v.service in service_names and v.traffic_split and
              v.traffic_split > 0 and v.id != args.version)

    # All versions that will stop receiving traffic.
    versions_to_migrate = list(filter(WillBeMigrated, all_versions))

    for version in versions_to_migrate:
      short_name = '{0}/{1}'.format(version.service, version.id)
      promoted_name = '{0}/{1}'.format(version.service, args.version)
      log.status.Print('Migrating all traffic from version '
                       '[{0}] to [{1}]'.format(
                           short_name, promoted_name))

    console_io.PromptContinue(cancel_on_no=True)

    errors = {}
    for service in sorted(set([v.service for v in versions_to_migrate])):
      allocations = {args.version: 1.0}
      try:
        operations_util.CallAndCollectOpErrors(
            client.SetTrafficSplit, service, allocations, shard_by='ip',
            migrate=True)
      except (operations_util.MiscOperationError) as err:
        errors[service] = str(err)

    if errors:
      error_string = ('Issues migrating all traffic of '
                      'service(s): [{0}]\n\n{1}'.format(
                          ', '.join(list(errors.keys())),
                          '\n\n'.join(list(errors.values()))))
      raise VersionsMigrateError(error_string)
